// Loader.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "resource.h"

// Import type library for IHandlerPtr
#include "ImportNotepadHandlerTlb.h"

#include "ObjectROTToken.h"	
#include "GetObjectFromROT.h"

// File contains functions to find/wait for window of specified class 
#include "GetTargetAppWindow.cpp"

const TCHAR g_szTarget[]   = _T("notepad.exe");	// target application
const TCHAR g_szSystem32[] = _T("system32");

const TCHAR g_szPluginName[] =	// plugin module name
#ifdef _DEBUG
	_T("NotepadPluginD.dll");
#else
	_T("NotepadPlugin.dll");
#endif

const TCHAR g_szKernel[]      = _T("Kernel32.dll");
const TCHAR g_szLoadLibrary[] = _T("LoadLibraryA");
	
TCHAR g_szPluginFullName[_MAX_PATH+1]; 


//
// Get process handle by name	
//
HANDLE GetProcessHandle(LPCTSTR szProcessName)
{
	const DWORD dwSize = 512;
	DWORD dwProcessID[dwSize];
	DWORD dwNeeded;
	HANDLE hProcess = NULL;

	// Construct full name of DLL to be injected
	DWORD dwPathLength = _MAX_PATH;
	TCHAR szPath[_MAX_PATH+1];
	GetCurrentDirectory( dwPathLength, szPath ); 
	_stprintf( g_szPluginFullName, _T("%s\\%s"), szPath, g_szPluginName );

	// Get the list of process identifiers
	if ( !EnumProcesses(dwProcessID, dwSize, &dwNeeded) )
		return NULL;
	
	// Calculate how many process identifiers were returned
	DWORD dwProcessesNum = dwNeeded / sizeof(DWORD);
	
	// Find handle of required process
	for (DWORD i=0; i<dwProcessesNum; i++)
	{
		hProcess = OpenProcess( PROCESS_ALL_ACCESS, FALSE, dwProcessID[i] );	
		if ( 0 != hProcess )
		{
			HMODULE hMod;			
			if ( EnumProcessModules( hProcess, &hMod, sizeof(hMod), &dwNeeded) )
			{
				TCHAR szCurProcessName[dwSize];
				GetModuleBaseName( hProcess, hMod, szCurProcessName, sizeof(szCurProcessName) );
				if ( !_tcscmp(szCurProcessName, szProcessName) )
					return hProcess;
			}
		}
		
		CloseHandle( hProcess );
	}

	return NULL;
}


//
// Inject DLL to target process. If target process is not running then start it.
//
int Load()
{
	DWORD   dwRemote = 0;					// remote thread ID
	HANDLE  hRemote  = 0;					// remote thread handle
	FARPROC pLoadLibrary  = NULL;			// pointer to LoadLibrary function
	LPVOID  pRemotePlugin = NULL;			// remote plugin filename buffer
	
	// Check whether object has been already registered with ROT
	CoInitialize( NULL );
	IHandlerPtr spHandler = NULL;
	HRESULT hr = GetObjectFromROT<IHandler>( (LPTSTR)g_szROTRegToken, &spHandler );
	if ( SUCCEEDED(hr) && NULL != spHandler )
		// Object has been already registered with ROT.
		// Assume that application is already running and "converted"
		return 0;	

	HANDLE hProcess = GetProcessHandle( g_szTarget );
	if ( 0 == hProcess )
	{
		// Target application is not running. Thus, it has to be started
		const DWORD dwTargetFullNameSize = _MAX_PATH;
		TCHAR szTargetPath[dwTargetFullNameSize];
		GetWindowsDirectory( szTargetPath, dwTargetFullNameSize );
		
		// Assume that target application is located in WindowsDir\system32 direstory
		TCHAR szFullTarget[dwTargetFullNameSize]; // full name of target application
		_stprintf( szFullTarget, _T("%s\\%s\\%s"), szTargetPath, g_szSystem32, g_szTarget );

		STARTUPINFO infoTargetInit;			// process startup structure
		PROCESS_INFORMATION infoTarget;		// process information structure
		memset( &infoTargetInit, 0, sizeof(infoTargetInit) );
		infoTargetInit.cb = sizeof( infoTargetInit );
		infoTargetInit.dwFlags = STARTF_USESHOWWINDOW;
		infoTargetInit.wShowWindow = SW_SHOWNORMAL;

		// Create target process
		CreateProcess( szFullTarget,				// application name
					   NULL,						// command line
					   NULL,						// process security
					   NULL,						// thread security
					   FALSE,						// inherit handles
					   CREATE_DEFAULT_ERROR_MODE,	// creation flags
					   NULL,						// environment
					   NULL,						// directory
					   &infoTargetInit,				// startup structure
					   &infoTarget					// information structure
					 );

		hProcess = infoTarget.hProcess;

		// Wait for main window of target application
		if ( WaitForWndOfClass(g_szTargetWndClass) == 0 )
			return -1;
	}

	// Target application is running
	DWORD dwPluginNameSize = _tcslen( g_szPluginFullName ) + 1;
	if ( 0 != hProcess )
		// Allocate buffer in target process address space
		pRemotePlugin = (LPTSTR)VirtualAllocEx( hProcess, 
											    NULL, 
											    dwPluginNameSize, 
											    MEM_COMMIT, 
											    PAGE_READWRITE 
											  );

	BOOL br = FALSE;
	if ( NULL != pRemotePlugin )
		// Copy plugin library filename into remote buffer
		br = WriteProcessMemory( hProcess, 
							     pRemotePlugin,
							     g_szPluginFullName,
							     dwPluginNameSize,
							     NULL 
							   );

	if ( br )
	{
		// Get address of LoadLibrary function
		HMODULE hModule = GetModuleHandle( g_szKernel );
		if ( 0 != hModule )
			pLoadLibrary = GetProcAddress( hModule, g_szLoadLibrary );
	}

	if ( NULL != pLoadLibrary )
		// Create remote thread to call LoadLibrary
		hRemote = CreateRemoteThread( hProcess,								// process
									  NULL,									// security
									  0,									// stack size
									  (LPTHREAD_START_ROUTINE)pLoadLibrary,	// function
									  pRemotePlugin,						// parameter
									  0,									// creation flags
									  NULL									// thread ID
									);

	if ( 0 != hRemote )
	{
		// Wait for remote thread to finish
		WaitForSingleObject( hRemote, INFINITE );
		GetExitCodeThread( hRemote, &dwRemote );
		CloseHandle( hRemote );
	}

	int ir = ( 0 != dwRemote ) ? /*Success!!*/0 : /*Failure??*/-1;

	if ( 0 != hProcess )
	{
		// Free resources
		VirtualFreeEx( hProcess, pRemotePlugin, 0, MEM_RELEASE );
		CloseHandle( hProcess );
	}

	return ir;
}


//
// Dll entry point
//
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	return Load();
}


